import asyncio
import os
import random
import string
from unittest import TestCase, mock

import pytest
import service
from asynctest import CoroutineMock
from simulation.game_runner import GameRunner

from .test_simulation.mock_worker_manager import MockWorkerManager


class MockGameState(object):
    turn_count = 0

    def serialize(self):
        return {"foo": "bar"}


class MockedSocketIOServer(mock.MagicMock):
    """ Decorator function that just returns the function. Needed because we decorate
        functions in the GameAPI."""

    def on(self, event):
        def decorator(func):
            def wrapper(*args, **kwargs):
                func(*args, **kwargs)

            return wrapper

        return decorator


class TestSocketIO:
    def setup_method(self, method):
        os.environ["GAME_ID"] = "1"
        self.environ = {"QUERY_STRING": "avatar_id=1&EIO=3&transport=polling&t=MJhoMgb"}
        self.game_api = self.create_game_api()
        self.mocked_mappings = self.game_api._socket_session_id_to_player_id
        self.sid = "".join(
            random.choice(
                string.ascii_uppercase + string.ascii_lowercase + string.digits
            )
            for _ in range(19)
        )

    def teardown_method(self, method):
        del os.environ["GAME_ID"]

    @mock.patch("docker.from_env")
    @mock.patch("service.app")
    def create_game_api(self, app, docker_from_env):
        game_runner = GameRunner(
            game_state_generator=lambda avatar_manager: MockGameState(),
            django_api_url="http://test",
            port="0000",
            worker_manager_class=MockWorkerManager,
        )
        return service.GameAPI(
            game_state=game_runner.game_state, worker_manager=game_runner.worker_manager
        )

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_socketio_emit_called(self, mocked_socketio, app):
        self.game_api.worker_manager.add_new_worker(1)

        await self.game_api.register_world_update_on_connect()(self.sid, self.environ)

        assert mocked_socketio.manager.emit.mockreturn_value.emit.assert_called_once

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server.emit", new_callable=CoroutineMock())
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_matched_session_id_to_avatar_id_mapping(
        self, mocked_socketio, mocked_emit, app
    ):
        assert len(self.mocked_mappings) == 0

        self.game_api.worker_manager.add_new_worker(1)
        await self.game_api.register_world_update_on_connect()(self.sid, self.environ),

        assert len(self.mocked_mappings) == 1
        assert self.sid in self.mocked_mappings
        assert int(self.mocked_mappings[self.sid]) == 1

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_no_match_session_id_to_avatar_id_mapping(self, mocked_socketio, app):
        self.environ["QUERY_STRING"] = "corrupted!@$%string123"

        assert len(self.mocked_mappings) == 0

        self.game_api.worker_manager.add_new_worker(1)

        await self.game_api.register_world_update_on_connect()(self.sid, self.environ),

        assert len(self.mocked_mappings) == 0
        assert not self.sid in self.mocked_mappings

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_send_updates_for_one_user(self, mocked_socketio, app):
        self.mocked_mappings[self.sid] = 1
        self.game_api.worker_manager.add_new_worker(self.mocked_mappings[self.sid])
        worker = self.game_api.worker_manager.player_id_to_worker[
            self.mocked_mappings[self.sid]
        ]
        worker.log = "Logs one"

        with mock.patch(
            "service.socketio_server.emit", new=CoroutineMock()
        ) as mocked_emit:
            await self.game_api.send_updates()

            game_state_call = mock.call("game-state", {"foo": "bar"}, room=self.sid)
            log_call = mock.call(
                "log", {"message": "Logs one", "turn_count": 0}, room=self.sid
            )

            mocked_emit.assert_has_calls([game_state_call, log_call], any_order=True)

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_no_logs_not_emitted(self, mocked_socketio, app):
        """ If there are no logs for an avatar, no logs should be emitted. """
        self.mocked_mappings[self.sid] = 1
        self.game_api.worker_manager.add_new_worker(self.mocked_mappings[self.sid])

        with mock.patch(
            "service.socketio_server.emit", new=CoroutineMock()
        ) as mocked_emit:
            await self.game_api.send_updates()

            mocked_emit.assert_called_once_with(
                "game-state", {"foo": "bar"}, room=self.sid
            )

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_empty_logs_not_emitted(self, mocked_socketio, app):
        """ If the logs are an empty sting, no logs should be emitted. """
        self.mocked_mappings[self.sid] = 1

        self.game_api.worker_manager.add_new_worker(self.mocked_mappings[self.sid])
        worker = self.game_api.worker_manager.player_id_to_worker[
            self.mocked_mappings[self.sid]
        ]
        worker.logs = ""

        with mock.patch(
            "service.socketio_server.emit", new=CoroutineMock()
        ) as mocked_emit:
            await self.game_api.send_updates()

            mocked_emit.assert_called_once_with(
                "game-state", {"foo": "bar"}, room=self.sid
            )

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_send_updates_for_multiple_users(self, mocked_socketio, app):
        self.mocked_mappings[self.sid] = 1
        self.mocked_mappings["differentsid"] = 2

        self.game_api.worker_manager.add_new_worker(self.mocked_mappings[self.sid])
        self.game_api.worker_manager.add_new_worker(
            self.mocked_mappings["differentsid"]
        )
        worker_one = self.game_api.worker_manager.player_id_to_worker[
            self.mocked_mappings[self.sid]
        ]
        worker_two = self.game_api.worker_manager.player_id_to_worker[
            self.mocked_mappings["differentsid"]
        ]
        worker_one.log = "Logs one"
        worker_two.log = "Logs two"

        with mock.patch(
            "service.socketio_server.emit", new=CoroutineMock()
        ) as mocked_emit:
            await self.game_api.send_updates()

            user_one_game_state_call = mock.call(
                "game-state", {"foo": "bar"}, room=self.sid
            )
            user_two_game_state_call = mock.call(
                "game-state", {"foo": "bar"}, room="differentsid"
            )
            user_one_log_call = mock.call(
                "log", {"message": "Logs one", "turn_count": 0}, room=self.sid
            )
            user_two_log_call = mock.call(
                "log", {"message": "Logs two", "turn_count": 0}, room="differentsid"
            )

            mocked_emit.assert_has_calls(
                [
                    user_one_game_state_call,
                    user_two_game_state_call,
                    user_one_log_call,
                    user_two_log_call,
                ],
                any_order=True,
            )

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_send_code_changed_flag(self, mocked_socketio, app):
        self.mocked_mappings[self.sid] = 1
        self.game_api.worker_manager.add_new_worker(self.mocked_mappings[self.sid])
        worker = self.game_api.worker_manager.player_id_to_worker[
            self.mocked_mappings[self.sid]
        ]
        worker.has_code_updated = True

        with mock.patch(
            "service.socketio_server.emit", new=CoroutineMock()
        ) as mocked_emit:
            await self.game_api.send_updates()

            user_game_state_call = mock.call(
                "game-state", {"foo": "bar"}, room=self.sid
            )
            user_game_code_changed_call = mock.call(
                "feedback-avatar-updated", {}, room=self.sid
            )

            mocked_emit.assert_has_calls(
                [user_game_state_call, user_game_code_changed_call], any_order=True
            )

    @pytest.mark.asyncio
    @mock.patch("service.app")
    @mock.patch("service.socketio_server", new_callable=MockedSocketIOServer)
    async def test_send_false_flag_not_sent(self, mocked_socketio, app):
        self.mocked_mappings[self.sid] = 1
        self.game_api.worker_manager.add_new_worker(self.mocked_mappings[self.sid])
        worker = self.game_api.worker_manager.player_id_to_worker[
            self.mocked_mappings[self.sid]
        ]
        worker.has_code_updated = False

        with mock.patch(
            "service.socketio_server.emit", new=CoroutineMock()
        ) as mocked_emit:
            await self.game_api.send_updates()

            mocked_emit.assert_called_once_with(
                "game-state", {"foo": "bar"}, room=self.sid
            )

    @pytest.mark.asyncio
    async def test_remove_session_id_on_disconnect(self):
        self.mocked_mappings[self.sid] = 1
        assert len(self.mocked_mappings) == 1
        assert self.sid in self.mocked_mappings
        assert self.mocked_mappings[self.sid] == 1

        await self.game_api.register_remove_session_id_from_mappings()(sid=self.sid)

        assert self.sid not in self.mocked_mappings
        assert len(self.mocked_mappings) == 0



import subprocess
import sys
import os
import stat
import errno
import platform
from subprocess import CalledProcessError
from urllib import urlretrieve, urlopen

BASE_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
TEST_BIN = os.path.join(BASE_DIR, 'test-bin')
OS = platform.system().lower()
FILE_SUFFIX = '.exe' if OS == 'windows' else ''
KUBECTL = os.path.join(TEST_BIN, 'kubectl%s' % FILE_SUFFIX)
MINIKUBE = os.path.join(TEST_BIN, 'minikube%s' % FILE_SUFFIX)
FNULL = open(os.devnull, 'w')


def log(message):
    sys.stderr.write(message + "\n")


def run_command(args, capture_output=False):
    try:
        if capture_output:
            return subprocess.check_output(args)
        else:
            subprocess.check_call(args)
    except CalledProcessError as e:
        log('Command failed with exit status %d: %s' % (e.returncode, ' '.join(args)))
        raise


def run_command_async(args, capture_output=False):
    if capture_output is True:
        p = subprocess.Popen(args, stdout=FNULL, stderr=subprocess.STDOUT)
    else:
        p = subprocess.Popen(args)
    return p


def create_test_bin():
    try:
        os.makedirs(TEST_BIN)
    except OSError as err:
        if err.errno != errno.EEXIST:
            raise


def binary_exists(filename):
    # Check if binary is callable on our path
    try:
        run_command([filename], True)
        return True
    except OSError:
        return False


def download_exec(url, dest):
    dest = urlretrieve(url, dest)[0]
    make_exec(dest)


def make_exec(file):
    current_stat = os.stat(file)
    os.chmod(file, current_stat.st_mode | stat.S_IEXEC)


def get_latest_github_version(repo):
    result = urlopen('https://github.com/%s/releases/latest' % repo)
    return result.geturl().split('/')[-1]



#!/usr/bin/env python

import ast
import asyncio
import json
import logging
import os
import signal
import sys
from urllib.parse import parse_qs

import aiohttp
import aiohttp_cors
import socketio
from aiohttp import web
from aiohttp_wsgi import WSGIHandler
from prometheus_client import make_wsgi_app

from activity_monitor import ActivityMonitor, StatusOptions
from authentication import initialize_game_token
from simulation import map_generator
from simulation.game_runner import GameRunner

app = web.Application()
cors = aiohttp_cors.setup(app)


activity_monitor = ActivityMonitor()
socketio_server = socketio.AsyncServer(async_handlers=True)

routes = web.RouteTableDef()

LOGGER = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class GameAPI(object):
    def __init__(self, game_state, worker_manager):
        self._socket_session_id_to_player_id = {}
        self.register_endpoints()
        self.worker_manager = worker_manager
        self.game_state = game_state

    def register_endpoints(self):
        self.register_player_data_view()
        self.register_world_update_on_connect()
        self.register_remove_session_id_from_mappings()
        self.register_healthcheck()
        app.add_routes(routes)

    def register_healthcheck(self):
        @routes.get("/game-{game_id}")
        async def healthcheck(request):
            return web.Response(text="HEALTHY")

        return healthcheck

    def register_player_data_view(self):
        @routes.get("/player/{player_id}")
        async def player_data(request: web.Request):
            player_id = int(request.match_info["player_id"])
            return web.json_response(
                {
                    "code": self.worker_manager.get_code(player_id),
                    "options": {},
                    "state": None,
                }
            )

        return player_data

    def register_world_update_on_connect(self):
        @socketio_server.on("connect")
        async def world_update_on_connect(sid, environ):
            query = environ["QUERY_STRING"]
            self._find_avatar_id_from_query(sid, query)
            activity_monitor.active_users = len(self._socket_session_id_to_player_id)
            await self.send_updates(on_connect=True)

        return world_update_on_connect

    def register_remove_session_id_from_mappings(self):
        @socketio_server.on("disconnect")
        async def remove_session_id_from_mappings(sid):
            LOGGER.info("Socket disconnected for session id:{}. ".format(sid))
            try:
                del self._socket_session_id_to_player_id[sid]
                activity_monitor.active_users = len(
                    self._socket_session_id_to_player_id
                )
            except KeyError:
                pass

        return remove_session_id_from_mappings

    async def send_updates(self, on_connect=False):
        player_id_to_worker = self.worker_manager.player_id_to_worker
        await self._send_have_avatars_code_updated(player_id_to_worker)
        await self._send_game_state()
        if not on_connect:
            await self._send_logs(player_id_to_worker)

    def _find_avatar_id_from_query(self, session_id, query_string):
        """
        :param session_id: Int with the session id
        :param query_string: String from the environment settings,
        usually located as the key 'QUERY_STRING'.
        """
        parsed_qs = parse_qs(query_string)

        try:
            avatar_id = int(parsed_qs["avatar_id"][0])
            self._socket_session_id_to_player_id[session_id] = avatar_id
        except ValueError:
            LOGGER.error("Avatar ID could not be casted into an integer")
        except KeyError:
            LOGGER.error("No avatar ID found. User may not be authorised ")
            LOGGER.error("query_string: " + query_string)

    async def _send_logs(self, player_id_to_workers):
        def should_send_logs(logs):
            return bool(logs)

        socket_session_id_to_player_id_copy = (
            self._socket_session_id_to_player_id.copy()
        )
        for sid, player_id in socket_session_id_to_player_id_copy.items():
            avatar_logs = player_id_to_workers[player_id].log
            if should_send_logs(avatar_logs):
                await socketio_server.emit(
                    "log",
                    {"message": avatar_logs, "turn_count": self.game_state.turn_count},
                    room=sid,
                )

    async def _send_game_state(self):
        serialized_game_state = self.game_state.serialize()
        socket_session_id_to_player_id_copy = (
            self._socket_session_id_to_player_id.copy()
        )
        for sid, player_id in socket_session_id_to_player_id_copy.items():
            await socketio_server.emit("game-state", serialized_game_state, room=sid)

    async def _send_have_avatars_code_updated(self, player_id_to_workers):
        socket_session_id_to_player_id_copy = (
            self._socket_session_id_to_player_id.copy()
        )
        for sid, player_id in socket_session_id_to_player_id_copy.items():
            if player_id_to_workers[player_id].has_code_updated:
                await socketio_server.emit("feedback-avatar-updated", {}, room=sid)


def create_runner(port):
    settings = json.loads(os.environ["settings"])
    generator = getattr(map_generator, settings["GENERATOR"])(settings)
    return GameRunner(
        game_state_generator=generator.get_game_state,
        django_api_url=os.environ.get(
            "GAME_API_URL", "http://localhost:8000/aimmo/api/games/"
        ),
        port=port,
    )


def run_game(port):
    game_runner = create_runner(port)

    def clean_token(self, app):
        game_runner.communicator.patch_token(data={"token": ""})

    asyncio.ensure_future(initialize_game_token(game_runner.communicator))

    app.on_shutdown.append(clean_token)
    app.on_shutdown.append(game_runner.communicator.close_session)

    game_api = GameAPI(
        game_state=game_runner.game_state, worker_manager=game_runner.worker_manager
    )
    game_runner.set_end_turn_callback(game_api.send_updates)
    asyncio.ensure_future(game_runner.run())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    host = sys.argv[1]

    socketio_server.attach(
        app, socketio_path=os.environ.get("SOCKETIO_RESOURCE", "socket.io")
    )

    if os.environ["WORKER"] == "local":
        port = int(os.environ["EXTERNAL_PORT"])
    else:
        port = int(sys.argv[2])

    run_game(port)

    wsgi_handler = WSGIHandler(make_wsgi_app())
    app.add_routes([web.get("/{path_info:metrics}", wsgi_handler)])

    logging.getLogger("socketio").setLevel(logging.ERROR)
    logging.getLogger("engineio").setLevel(logging.ERROR)
    LOGGER.info("starting the server")
    web.run_app(app, host=host, port=port)



import asyncio
from unittest import TestCase

from simulation.location import Location
from simulation.interactables.pickups import InvulnerabilityPickup

from .mock_world import MockWorld


class TestInvulnerabilityPickupsAndEffects(TestCase):
    def setUp(self):
        """
        Mock a game for each test individually. MockWorld() will set up a:
        avatar manager, game state, turn manager and a map generator.
        """
        self.game = MockWorld()
        self.game.simulation_runner.add_avatar(1, Location(0, 0))
        self.cell = self.game.game_state.world_map.get_cell(Location(1, 0))

    def test_invulnerability_pickups_increase_resistance_of_avatar(self):
        """
        Avatar spawns at the origin (0,0) and should have a resistance of 0. Moves
        EAST to (1,0) and should automatically pick up the pickup and get the
        effect.
        """
        self.cell.interactable = InvulnerabilityPickup(self.cell)
        self.assertEqual(self.game.avatar_manager.get_avatar(1).resistance, 0)
        self.assertEqual(
            self.cell.interactable.serialize(),
            {
                "type": "invulnerability",
                "location": {"x": self.cell.location.x, "y": self.cell.location.y},
            },
        )

        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self.game.simulation_runner.run_single_turn(
                self.game.avatar_manager.get_player_id_to_serialized_action()
            )
        )

        self.assertEqual(self.cell.avatar, self.game.avatar_manager.get_avatar(1))
        self.assertEqual(self.cell.avatar.resistance, 1000)
        self.assertEqual(len(self.cell.avatar.effects), 1)

    def test_invulnerability_pickups_can_increase_resistance_to_2000(self):
        """
        Checks if the pickup can be applied twice. First moved from ORIGIN to 1,0 ->
        then picks up the pickup, and moves to 2,0 to do the same.
        """
        loop = asyncio.get_event_loop()
        self.cell.interactable = InvulnerabilityPickup(self.cell)
        loop.run_until_complete(
            self.game.simulation_runner.run_single_turn(
                self.game.avatar_manager.get_player_id_to_serialized_action()
            )
        )
        self.assertEqual(self.cell.avatar, self.game.avatar_manager.get_avatar(1))
        self.assertEqual(self.cell.avatar.resistance, 1000)

        self.cell = self.game.game_state.world_map.get_cell(Location(2, 0))
        self.cell.interactable = InvulnerabilityPickup(self.cell)
        loop.run_until_complete(
            self.game.simulation_runner.run_single_turn(
                self.game.avatar_manager.get_player_id_to_serialized_action()
            )
        )

        self.assertEqual(self.cell.avatar.resistance, 2000)
        self.assertEqual(self.cell.avatar, self.game.avatar_manager.get_avatar(1))



import os
import mock

from unittest import TestCase
from urllib.parse import urlparse

from simulation.workers.local_worker import LocalWorker
from simulation.worker_manager import WorkerManager


class TestLocalWorker(TestCase):
    @mock.patch("docker.from_env")
    def test_local_worker_ports_do_not_conflict(self, docker_from_env):
        os.environ["GAME_ID"] = "1"
        LocalWorker._init_port_counter()
        worker_manager1 = WorkerManager()
        worker_manager1.add_new_worker(1)
        local_worker1 = worker_manager1.player_id_to_worker[1]
        url1 = urlparse(local_worker1.url)

        os.environ["GAME_ID"] = "2"
        LocalWorker._init_port_counter()
        worker_manager2 = WorkerManager()
        worker_manager2.add_new_worker(1)
        local_worker2 = worker_manager2.player_id_to_worker[1]
        url2 = urlparse(local_worker2.url)

        self.assertEqual(url1.port, 11989)
        self.assertEqual(url2.port, 21989)

    @mock.patch("docker.from_env")
    def test_local_worker_in_the_same_game_do_not_have_port_conflicts(
        self, docker_from_env
    ):
        os.environ["GAME_ID"] = "1"
        LocalWorker._init_port_counter()
        worker_manager = WorkerManager()
        worker_manager.add_new_worker(1)
        worker_manager.add_new_worker(2)
        local_worker1 = worker_manager.player_id_to_worker[1]
        local_worker2 = worker_manager.player_id_to_worker[2]

        url1 = urlparse(local_worker1.url)
        url2 = urlparse(local_worker2.url)

        self.assertEqual(url1.port, 11989)
        self.assertEqual(url2.port, 11990)



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

import versioneer

setup(
    name="aimmo",
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "django >= 1.8.3, <= 1.10.8",
        "django-autoconfig >= 0.3.6, < 1.0.0",
        "django-forms-bootstrap",
        "django-js-reverse",
        "djangorestframework>=3.8.2, < 3.9.0",
        "eventlet",
        "flask",
        "flask-socketio",
        "requests",
        "six",
        "hypothesis",
        "flask-cors >= 3.0, < 3.1",
        "psutil >= 5.4, < 5.5",
        "RestrictedPython == 4.0.b7",
    ],
    tests_require=[
        "httmock",
        "mock == 2.0.0",
        "docker >= 3.5, < 3.6",
        "kubernetes == 5.0.0",
        "PyYAML == 4.2b1",
    ],
    version=versioneer.get_version(),
    zip_safe=False,
)



import asyncio
import math
from unittest import TestCase

from hypothesis import assume, given
from hypothesis import strategies as st
from simulation.location import Location
from simulation.interactables.pickups import DamageBoostPickup
from simulation.interactables.effects import DAMAGE_BOOST_DEFAULT

from .mock_world import MockWorld


class TestDamagePickupsAndEffects(TestCase):
    def setUp(self):
        """
        Mock a game for each test individually. MockWorld() will set up a:
        avatar manager, game state, turn manager and a map generator.
        """
        self.game = MockWorld()
        self.game.simulation_runner.add_avatar(1, Location(0, 0))
        _avatar_spawn_cell = self.game.game_state.world_map.get_cell(Location(0, 0))
        self.initial_attack_strength = _avatar_spawn_cell.avatar.attack_strength
        self.cell = self.game.game_state.world_map.get_cell(Location(1, 0))
        self.loop = asyncio.get_event_loop()

    def test_damage_boost_pickup_can_be_picked_up_default(self):
        pickup_created = DamageBoostPickup(self.cell)
        self.cell.interactable = pickup_created
        self.assertEqual(
            self.cell.interactable.serialize(),
            {
                "type": "damage_boost",
                "location": {"x": self.cell.location.x, "y": self.cell.location.y},
            },
        )
        self.loop.run_until_complete(
            self.game.simulation_runner.run_single_turn(
                self.game.avatar_manager.get_player_id_to_serialized_action()
            )
        )

        self.assertEqual(self.cell.avatar, self.game.avatar_manager.get_avatar(1))
        self.assertEqual(len(self.game.avatar_manager.get_avatar(1).effects), 1)
        damage_boost_effect = self.game.avatar_manager.get_avatar(1).effects.pop()
        self.assertTrue(isinstance(damage_boost_effect, pickup_created.effects[0]))

    def test_damage_boost_increases_attack_strength_with_default_integer(self):
        """
        Damage boost with no value parameter provided (ie. default).
        """
        pickup_created = DamageBoostPickup(self.cell)
        self.cell.interactable = pickup_created

        self.loop.run_until_complete(
            self.game.simulation_runner.run_single_turn(
                self.game.avatar_manager.get_player_id_to_serialized_action()
            )
        )

        self.assertTrue(
            self.cell.avatar.attack_strength,
            self.initial_attack_strength + DAMAGE_BOOST_DEFAULT,
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-04-04 14:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aimmo', '0008_default_public_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='game',
            name='status',
            field=models.CharField(choices=[(b'r', b'running'), (b's', b'stopped'), (b'p', b'paused')], default=b'r', max_length=1),
        ),
    ]
